import click
from rich.syntax import Syntax
from rich.table import Table
from pygments.styles import get_all_styles


@click.group(name='themes', help='Commands for working with themes.')
def themes_group():
    pass


@themes_group.command(name='list', help='List all available themes.')
@click.pass_obj
def list_themes(console):

    styles = list(get_all_styles())

    table = Table(title='Console themes')
    table.add_column('Themes', style='cyan', no_wrap=True)

    for i in styles:
        table.add_row(i)

    console.print(table)


@themes_group.command(name='show', help='Display examples of available themes')
@click.option('--name',
              '-n',
              default=None,
              help='''
              Display a named theme
              '''
              )
@click.pass_obj
def show_themes(console, name):

    styles = list(get_all_styles())

    data = '''
        def test_function(i):
            print(i)
    '''.strip()

    if name is not None:

        if name not in styles:
            raise click.ClickException(f'{name} is not a valid theme. Use catz themes list to view available themes')

        syntax_params = dict(
            code=data,
            lexer_name='python',
            theme=name,
            line_numbers=True
        )
        console.print(f'Theme: {name}')
        console.print(Syntax(**syntax_params))

    else:
        for i in styles:
            syntax_params = dict(
                code=data,
                lexer_name='python',
                theme=i,
                line_numbers=True
            )

            console.print(f'Theme: {i}')
            console.print(Syntax(**syntax_params))



import sys
import click
from rich.syntax import Syntax
from .util import (
    get_lexer_from_name,
    get_content_from_url,
    get_content_from_file,
    is_url
)


class HandleRangeInput(click.Option):
    """
    Override to handle different types of line number inputs for
    --highlight / -hl.

    Accepts a csv of numbers (1,2,6) or a range (1-6).
    """
    def consume_value(self, ctx, opts):
        value = super().consume_value(ctx, opts)
        highlight_exception_base = 'Invalid value for --highlight / -hl:'

        if value is not None:

            try:

                if '-' in value:
                    input_list = value.split('-')

                    if len(input_list) > 2:
                        raise click.ClickException(
                            f'{highlight_exception_base} Could not convert {value} to a valid range')

                    input_list = list(map(int, input_list))

                    if input_list[0] > input_list[1]:
                        raise click.ClickException(
                            f'{highlight_exception_base} {input_list[0]} is greater than {input_list[1]}')

                    value = list(range(input_list[0], input_list[1]+1))

                    if len(value) == 0:
                        value = input_list[0]

                else:
                    value = list(map(int, value.split(',')))

            except ValueError as e:
                raise click.ClickException(
                    f'{highlight_exception_base} {e} is not a valid integer range')

        return value


@click.command(name='get',
               help='Perform syntax highlighting on raw text from a local file or a url.',
               no_args_is_help=True
               )
@click.argument('path')
@click.option('--theme',
              '-t',
              default='native',
              envvar='CATZ_THEME',
              help='''Override the default syntaxt highlighting theme.
              You can use catz themes list to view a list of available themes.'''
              )
@click.option('--lexer',
              '-l',
              default=None,
              envvar='CATZ_LEXER',
              help='''Override the lexer used when applying syntax highlighting.
              You can use catz lexers list to view a list of available lexers.'''
              )
@click.option('--highlight',
              '-hl',
              default=None,
              cls=HandleRangeInput,
              help='''Highlight specific lines in the parsed file.
              Accepts a comma separated list of line numbers.
              '''
              )
@click.option('--passthru',
              '-p',
              is_flag=True,
              envvar='CATZ_PASSTHRU',
              help='''Pass the content of the file directly to stdout with no lexer applied.'''
              )
@click.pass_obj
def get(console, path, theme, lexer, highlight, passthru):

    data, lexer_name = get_content_from_url(
        path) if is_url(path) else get_content_from_file(path)

    if passthru:
        print(data, file=sys.stdout)
        return

    if lexer is not None:
        lexer_name = get_lexer_from_name(lexer)

    syntax_params = dict(
        code=data,
        lexer_name=lexer_name,
        theme=theme,
        line_numbers=True
    )

    if highlight is not None:
        syntax_params['highlight_lines'] = set(highlight)

    console.print(Syntax(**syntax_params))



import os
import urllib
import click
from urllib import parse, request
from pathlib import Path
from pygments.lexers import (
    get_lexer_for_filename,
    get_lexer_by_name,
    get_lexer_for_mimetype,
    ClassNotFound
)


def __get_lexer_from_mimetype(mimetype):
    """
    Internal function to gracefully search for a lexer
    by mime type.
    """
    try:
        lexer = get_lexer_for_mimetype(mimetype)
        return lexer.name
    except ClassNotFound:
        pass


def __get_lexer_from_filename(filename):
    """
    Internal function to gracefully search for a lexer
    by filename.
    """
    try:
        lexer = get_lexer_for_filename(filename)
        return lexer.name
    except ClassNotFound:
        pass


def flatten(input):
    """
    Flattern a list.
    """
    return ','.join([x for x in input])


def is_url(input):
    """
    Test if the given input is a valid URL
    """
    url = parse.urlparse(input)
    if url.netloc and url.scheme:
        return True


def get_content_from_url(url):
    """
    Retrieve content from a url.

    Returns the raw data and a lexer if one can be found.
    """
    VALID_PROTOCOLS = ['http', 'https']
    p = parse.urlparse(url).scheme
    if p not in VALID_PROTOCOLS:
        raise click.ClickException(f'{p} is not a valid http protocol.')

    try:
        with request.urlopen(url) as conn:
            content = conn.read().decode('utf-8')
            mime_type = conn.headers['content-type'].split(';')
            lexer = __get_lexer_from_mimetype(mime_type)
            return content, lexer

    except urllib.error.HTTPError as e:
        raise click.ClickException(e)


def get_content_from_file(file):
    """
    Retrieve content from a file.

    Returns the raw data and a lexer if one can be found.
    """
    try:

        resolved_path = os.path.expanduser(
            file) if file.startswith('~') else Path(file).resolve()

        with open(resolved_path, encoding='utf-8', errors='ignore') as file:
            data = file.read()
            filename = file.name
        lexer = __get_lexer_from_filename(filename)
        return data, lexer

    except OSError as e:
        raise click.ClickException(f'OSError: {e}')

    except FileNotFoundError:
        raise click.ClickException(f'FileNotFound: {file}')

    except UnicodeEncodeError as e:
        raise click.ClickException(f'Could not handle file encoding: {e}')


def get_lexer_from_name(name):
    """
    Get lexer by name
    """
    try:
        lexer = get_lexer_by_name(name)
        return lexer.name
    except ClassNotFound:
        pass
